/* LCD Include-Datei
 *
 * Das LCD ist folgendermaen angeschlossen:
 * PD4     = RS
 * PD5     = Enable
 * PD0-PD3 = D4-D7
 * R/W ist n.c.
 *
 * 03/2005 Florian Schffer, http://www.blafusel.de
 *
 */

#include <avr/io.h>

#define F_CPU 7373200UL    //
#include <avr/delay.h>            

#define delay(us)  _delay_loop_2 (((F_CPU/4000)*us)/1000)		// wartet s

// Enable-Leitung toggeln
void lcd_flash_e ()
{
	PORTD = PORTD | ( 1<<DDD5 ) ;		// Enable = HIGH
	//PORTD = PORTD | 0b00100000;
	
	
	delay(1);	 // 1						// kurz warten, damit Flanke entsteht
	PORTD = PORTD & ~( 1<<DDD5 ) ;		// Enable = LOW
	//PORTD = PORTD & 0b11011111;
	
	
	delay(64);	// 64						// Warten, bis der durch Enable angelegte Befehl hoffentlich verarbeitet wurde
}

// Kommando oder Zeichen an Display senden: rs=0 => Kommando, rs=1 => Zeichen
void lcd_write (uint8_t data, uint8_t rs)   
{
	uint8_t dataBits ;
  
    if (rs)    				// write data (RS=1, RW=0) 
       dataBits=0x10;		// RS liegt an Pin 4 = B 0001 0000 = H 10
    else     				// write instruction (RS=0, RW=0) 
       dataBits=0; 			
	
    PORTD = dataBits |(data>>4);		// output high nibble first, zzgl. Zustand fr RS-Leitung
	lcd_flash_e ();    
    PORTD = dataBits | (data&0x0F);	// output low nibble, zzgl. RS-Leitung
	lcd_flash_e ();
	delay(2000);
}

// Display loeschen
void lcd_cls ()   
{
	lcd_write(0x02,0);   	// B 0000 0010 => Display loeschen
	delay(2000);			// dauert eine Weile, Wert ausprobiert
	lcd_write(0x01,0);   	// B 0000 0001 => Cursor Home
	delay(2000);			// dauert eine Weile, Wert ausprobiert
}

// Zeichenausgabe
void lcd_writechar ( char zeichen)   
{
  lcd_write (zeichen, 1);   
}

// gibt eine Zeichenkette auf dem LCD aus
void lcd_writetext ( char *text)    
{
  uint8_t i = 0;
  while (text[i]!=0) 
  {
    lcd_writechar(text[i]);
    i++;
  }
}

// Zeilenwechsel
void lcd_gotoline (uint8_t zeile)    
{
  if (zeile == 1) lcd_write(0x80,0);   // B 1000 0000 => DD-RAM Adress Set 1. Zeile/1.Spalte 
  if (zeile == 2) lcd_write(0xC0,0);   // B 1100 0000 => DD-RAM Adress Set 2. Zeile/1.Spalte (B x100 0000 = H 40)
}

// Cursorpositionieren (Zeile, Spalte)
void lcd_gotopos (uint8_t zeile, uint8_t spalte)    
{
  if (zeile == 1) lcd_write(0x80+spalte-1,0);   // DD-RAM Adress 1. Zeile + Spalte
  if (zeile == 2) lcd_write(0xC0+spalte-1,0);   // DD-RAM Adress 2. Zeile + Spalte 
}

// Display initialisieren. Einmal als erstes aufrufen
void lcd_ini ()
{
	DDRD = 0x3F;		// setze Portrichtung (1 = Ausgang): 0011 1111 
	PORTD=0x00;			// alle Leitungen LOW

	delay(10000);		// 10ms warten bis LCD wirklich bereit (max. Wert lt. Datenblatt)

	lcd_write(0x38,0); 		// B 0010 1000 => 8-Bit-Modus aktivieren, Ist wichtig, falls LCD schon im 4-Bit Modus war und dann 
							// nach einem Programm-Reset vergeblich versucht wrde, den 4-Bit Modus erneut zu aktivieren
	lcd_write(0x2,0);		// B 0000 0010 => mit 8-Bit-Command in 4-Bit-Modus umschalten 

	lcd_flash_e ();			// Enable

	delay(1000);			// sicherheitshalber warten

	// ab jetzt 4-Bit-Modus
	lcd_write(0x28,0);     // B 0010 1000 => Function Set: 4Bit (kein 8-Bit Bus), zweizeiliges Display, 5x7 Dots/Zeichen (kein 5x10)
	lcd_write(0x0C,0);     // B 0000 1100 => Display On/Off: Display ein, Cursor aus, Blinken aus
	lcd_write(0x06,0);     // B 0000 0110 => Entry Mode Set: DD-RAM autom. inkr., Cursor bewegen
	
	lcd_cls();				// Display lschen
}


